@@ -1,6 +1,5 @@
-from enum import Enum
+from enum import Enum, auto
 
-from ..lib.auto_repl import auto
 
 class TokenTypes(Enum):
     ALL = auto()
@@ -21,21 +20,21 @@ class TokenTypes(Enum):
     GROUP = auto()
     IF = auto()
     IN = auto()
-    LIGHTS = auto()
     LITERAL_STRING = auto()
     LOCATION = auto()
     LOGICAL = auto()
     NAME = auto()
-    NO_TOKEN = auto()
+    NULL = auto()
     NUMBER = auto()
     OFF = auto()
     ON = auto()
     OR = auto()
-    POWER = auto()
     PRINT = auto()
+    PRINTF = auto()
     PRINTLN = auto()
     PAUSE = auto()
     RAW = auto()
+    RGB = auto()
     REGISTER = auto()
     REPEAT = auto()
     SET = auto()
@@ -49,25 +48,10 @@ class TokenTypes(Enum):
     WAIT = auto()
     ZONE = auto()
 
-    @classmethod
-    def commands(cls):
-        return (TokenTypes.ASSIGN,
-                TokenTypes.GET,
-                TokenTypes.OFF,
-                TokenTypes.ON,
-                TokenTypes.POWER,
-                TokenTypes.PAUSE,
-                TokenTypes.REGISTER,
-                TokenTypes.SET,
-                TokenTypes.UNITS,
-                TokenTypes.WAIT)
-
-    def is_command(self):
-        return self in TokenTypes.commands()
-
-    def is_printable(self):
+    def is_executable(self):
         return self in (
-            TokenTypes.EXPRESSION, TokenTypes.LITERAL_STRING, TokenTypes.NAME,
-            TokenTypes.NUMBER, TokenTypes.REGISTER, TokenTypes.TIME_PATTERN,
-            TokenTypes.UNKNOWN
-        )
+            TokenTypes.ASSIGN, TokenTypes.BREAKPOINT,
+            TokenTypes.GET, TokenTypes.IF, TokenTypes.OFF, TokenTypes.ON,
+            TokenTypes.PRINT, TokenTypes.PRINTF, TokenTypes.PRINTLN,
+            TokenTypes.PAUSE, TokenTypes.REGISTER, TokenTypes.REPEAT,
+            TokenTypes.SET, TokenTypes.UNITS, TokenTypes.WHILE, TokenTypes.WAIT)
