@@ -1,24 +1,26 @@
-from enum import Enum
-
-from bardolph.lib.auto_repl import auto
+from enum import Enum, auto
+import logging
 
 class Register(Enum):
+    BLUE = auto()
     BRIGHTNESS = auto()
     DURATION = auto()
     FIRST_ZONE = auto()
+    GREEN = auto()
     HUE = auto()
     LAST_ZONE = auto()
     KELVIN = auto()
     NAME = auto()
     OPERAND = auto()
     POWER = auto()
+    RED = auto()
     RESULT = auto()
     SATURATION = auto()
     TIME = auto()
     UNIT_MODE = auto()
 
-    @classmethod
-    def from_string(cls, name):
+    @staticmethod
+    def from_string(name):
         upper = name.upper()
         return getattr(Register, upper) if hasattr(Register, upper) else None
 
@@ -41,7 +43,6 @@ class OpCode(Enum):
     NOP = auto()
     OP = auto()
     OUT = auto()
-    OUTQ = auto()
     PARAM = auto()
     PAUSE = auto()
     POP = auto()
@@ -94,6 +95,11 @@ class Operand(Enum):
     MZ_LIGHT = auto()
     NULL = auto()
 
+class IoOp(Enum):
+    PRINT = auto()
+    PRINTF = auto()
+    REGISTER = auto()
+
 class SetOp(Enum):
     """ Used with TimePattern """
     INIT = auto()
